/********************************************************************************
*                                                                               *
*                        T o o l b a r   W i d g e t                            *
*                                                                               *
*********************************************************************************
* Copyright (C) 2000 by Jeroen van der Zijp.   All Rights Reserved.             *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
*********************************************************************************
* $Id: FXToolbar.cpp,v 1.37 2000/08/28 14:40:07 jeroen Exp $                    *
********************************************************************************/
#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXObject.h"
#include "FXDict.h"
#include "FXRegistry.h"
#include "FXAccelTable.h"
#include "FXApp.h"
#include "FXId.h"
#include "FXDC.h"
#include "FXDCWindow.h"
#include "FXDrawable.h"
#include "FXWindow.h"
#include "FXFrame.h"
#include "FXComposite.h"
#include "FXPacker.h"
#include "FXToolbar.h"
#include "FXShell.h"
#include "FXTopWindow.h"
#include "FXToolbarGrip.h"

  
/*
  Notes:
  - Wrapping algorithm not OK yet; use DP.
  - Divider widgets and special layout ideas:
    
      +---------+---------------+
      | 1 2 3 4 | 5 6 7         |
      +---------+---------------+
      
    after wrap:
    
      +-----+-----+
      | 1 2 | 5 6 |  Clue: use D.P.
      | 3 4 | 7   |
      +-----+-----+
      
    wrap:
    
      +-------------+
      | L L L L   R |
      +-------------+
      
    to:
    
      +---------+
      | L L L R |
      +---------+
      | L       |
      +---------+
      
*/


// Docking side
#define LAYOUT_SIDE_MASK (LAYOUT_SIDE_LEFT|LAYOUT_SIDE_RIGHT|LAYOUT_SIDE_TOP|LAYOUT_SIDE_BOTTOM)

/*******************************************************************************/

// Map
FXDEFMAP(FXToolbar) FXToolbarMap[]={
  FXMAPFUNC(SEL_FOCUS_PREV,0,FXToolbar::onFocusLeft),
  FXMAPFUNC(SEL_FOCUS_NEXT,0,FXToolbar::onFocusRight),
  };


// Object implementation
FXIMPLEMENT(FXToolbar,FXPacker,FXToolbarMap,ARRAYNUMBER(FXToolbarMap))

  
// Deserialization
FXToolbar::FXToolbar(){
  drydock=NULL;
  wetdock=NULL;
  }


// Make a floatable toolbar
FXToolbar::FXToolbar(FXComposite* p,FXComposite* q,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
  drydock=p;
  wetdock=q;
  }


// Make a non-floatable toolbar
FXToolbar::FXToolbar(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
  drydock=NULL;
  wetdock=NULL;
  }


// Change toolbar orientation
void FXToolbar::setDockingSide(FXuint side){

  // Just the side
  side&=LAYOUT_SIDE_MASK;

  // Adjust options to new orientation
  if(side&LAYOUT_SIDE_LEFT){            // Making it vertical
    if(!(options&LAYOUT_SIDE_LEFT)){    // Was horizontal
      if((options&LAYOUT_RIGHT) && (options&LAYOUT_CENTER_X)) side|=LAYOUT_FIX_Y;
      else if(options&LAYOUT_RIGHT) side|=LAYOUT_BOTTOM;
      else if(options&LAYOUT_CENTER_X) side|=LAYOUT_CENTER_Y;
      if(options&LAYOUT_FILL_X) side|=LAYOUT_FILL_Y;
      }
    else{                               // Was vertical already
      side|=(options&(LAYOUT_BOTTOM|LAYOUT_CENTER_Y|LAYOUT_FILL_Y));
      }
    }
  else{                                 // Making it horizontal
    if(options&LAYOUT_SIDE_LEFT){       // Was vertical
      if((options&LAYOUT_BOTTOM) && (options&LAYOUT_CENTER_Y)) side|=LAYOUT_FIX_X;
      else if(options&LAYOUT_BOTTOM) side|=LAYOUT_RIGHT;
      else if(options&LAYOUT_CENTER_Y) side|=LAYOUT_CENTER_X;
      if(options&LAYOUT_FILL_Y) side|=LAYOUT_FILL_X;
      }
    else{
      side|=(options&(LAYOUT_RIGHT|LAYOUT_CENTER_X|LAYOUT_FILL_X));
      }
    }

  // Simply preserve these options
  side|=(options&(LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT));

  // Update the layout
  setLayoutHints(side);
  }


// Get toolbar orientation
FXuint FXToolbar::getDockingSide() const { 
  return (options&LAYOUT_SIDE_MASK)!=0; 
  }


// Compute minimum width based on child layout hints
int FXToolbar::getDefaultWidth(){
  register FXint w,wcum,wmax,mw,n;
  register FXWindow* child;
  register FXuint hints;
  wcum=wmax=n=0;
  if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
  for(child=getFirst(); child; child=child->getNext()){
    if(child->shown()){
      hints=child->getLayoutHints();
      if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) w=child->getDefaultWidth();
      else if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
      else if(options&PACK_UNIFORM_WIDTH) w=mw;
      else w=child->getDefaultWidth();
      if(wmax<w) wmax=w;
      wcum+=w; 
      n++;
      }
    }
  if(!(options&LAYOUT_SIDE_LEFT)){
    if(n>1) wcum+=(n-1)*hspacing;
    wmax=wcum;
    }
  return padleft+padright+wmax+(border<<1);
  }
  

// Compute minimum height based on child layout hints
int FXToolbar::getDefaultHeight(){
  register FXint h,hcum,hmax,mh,n;
  register FXWindow* child;
  register FXuint hints;
  hcum=hmax=n=0;
  if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
  for(child=getFirst(); child; child=child->getNext()){
    if(child->shown()){
      hints=child->getLayoutHints();
      if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) h=child->getDefaultHeight();
      else if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
      else if(options&PACK_UNIFORM_HEIGHT) h=mh;
      else h=child->getDefaultHeight();
      if(hmax<h) hmax=h;
      hcum+=h; 
      n++;
      }
    }
  if(options&LAYOUT_SIDE_LEFT){
    if(n>1) hcum+=(n-1)*vspacing;
    hmax=hcum;
    }
  return padtop+padbottom+hmax+(border<<1);
  }


// Return width for given height
FXint FXToolbar::getWidthForHeight(FXint givenheight){
  FXint wtot,wmax,hcum,w,h,mw,mh,space,ngalleys;
  FXWindow* child;
  FXuint hints;
  wtot=wmax=hcum=ngalleys=0;
  space=givenheight-padtop-padbottom-(border<<1);
  if(space<1) space=1;
  if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
  if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
  for(child=getFirst(); child; child=child->getNext()){
    if(child->shown()){
      hints=child->getLayoutHints();
      if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) w=child->getDefaultWidth();
      else if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
      else if(options&PACK_UNIFORM_WIDTH) w=mw;
      else w=child->getDefaultWidth();
      if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) h=child->getDefaultHeight();
      else if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
      else if(options&PACK_UNIFORM_HEIGHT) h=mh;
      else h=child->getDefaultHeight();
      if(hcum+h>space) hcum=0;
      if(hcum==0) ngalleys++;
      hcum+=h+vspacing;
      if(wmax<w) wmax=w;
      }
    }
  wtot=wmax*ngalleys;
  return padleft+padright+wtot+(border<<1);
  }


// Return height for given width
FXint FXToolbar::getHeightForWidth(FXint givenwidth){
  FXint htot,hmax,wcum,w,h,mw,mh,space,ngalleys;
  FXWindow* child;
  FXuint hints;
  htot=hmax=wcum=ngalleys=0;
  space=givenwidth-padleft-padright-(border<<1);
  if(space<1) space=1;
  if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
  if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
  for(child=getFirst(); child; child=child->getNext()){
    if(child->shown()){
      hints=child->getLayoutHints();
      if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) w=child->getDefaultWidth();
      else if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
      else if(options&PACK_UNIFORM_WIDTH) w=mw;
      else w=child->getDefaultWidth();
      if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) h=child->getDefaultHeight();
      else if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
      else if(options&PACK_UNIFORM_HEIGHT) h=mh;
      else h=child->getDefaultHeight();
      if(wcum+w>space) wcum=0;
      if(wcum==0) ngalleys++;
      wcum+=w+hspacing;
      if(hmax<h) hmax=h;
      }
    }
  htot=hmax*ngalleys;
  return padtop+padbottom+htot+(border<<1);
  }


// Recalculate layout
void FXToolbar::layout(){
  FXint galleyleft,galleyright,galleytop,galleybottom,galleywidth,galleyheight;
  FXint tleft,tright,ttop,bleft,bright,bbottom;
  FXint ltop,lbottom,lleft,rtop,rbottom,rright;
  FXWindow *child;
  FXint x,y,w,h,mw,mh;
  FXuint hints;

  //FXTRACE((150,"%s::layout\n",getClassName()));

  // Get maximum child size
  if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
  if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();

  // Vertical toolbar
  if(options&LAYOUT_SIDE_LEFT){
    galleywidth=0;
    for(child=getFirst(); child; child=child->getNext()){
      if(child->shown()){
        hints=child->getLayoutHints();
        if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) w=child->getDefaultWidth();
        else if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
        else if(options&PACK_UNIFORM_WIDTH) w=mw;
        else w=child->getDefaultWidth();
        if(galleywidth<w) galleywidth=w;
        }
      }
    galleyleft=border+padleft;
    galleyright=width-border-padright;
    galleytop=border+padtop;
    galleybottom=height-border-padbottom;
    //FXTRACE((100,"vert galleyleft=%d galleyright=%d galleytop=%d galleybottom=%d galleywidth=%d\n",galleyleft,galleyright,galleytop,galleybottom,galleywidth));
    tleft=galleyleft;
    tright=galleyleft+galleywidth;
    ttop=galleytop;
    bright=galleyright;
    bleft=galleyright-galleywidth;
    bbottom=galleybottom;
    for(child=getFirst(); child; child=child->getNext()){
      if(child->shown()){
        hints=child->getLayoutHints();
        if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))){
          w=galleywidth;
          h=child->getDefaultHeight();
          }
        else{
          if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
          else if(options&PACK_UNIFORM_WIDTH) w=mw;
          else w=child->getDefaultWidth();
          if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
          else if(options&PACK_UNIFORM_HEIGHT) h=mh;
          else h=child->getDefaultHeight();
          }
        if(hints&LAYOUT_BOTTOM){
          if(bbottom-h<galleytop && bbottom!=galleybottom){
            bright=bleft;
            bleft-=galleywidth;
            bbottom=galleybottom;
            }
          y=bbottom-h;
          bbottom-=(h+vspacing);
          x=bleft+(galleywidth-w)/2;
          //FXTRACE((100,"vert bottom: x=%d y=%d w=%d h=%d\n",x,y,w,h));
          }
        else{
          if(ttop+h>galleybottom && ttop!=galleytop){
            tleft=tright;
            tright+=galleywidth;
            ttop=galleytop;
            }
          y=ttop;
          ttop+=(h+vspacing);
          x=tleft+(galleywidth-w)/2;
          //FXTRACE((100,"vert top   : x=%d y=%d w=%d h=%d\n",x,y,w,h));
          }
        child->position(x,y,w,h);
        }
      }
    }
  
  // Horizontal toolbar
  else{
    galleyheight=0;
    for(child=getFirst(); child; child=child->getNext()){
      if(child->shown()){
        hints=child->getLayoutHints();
        if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))) h=child->getDefaultHeight();
        else if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
        else if(options&PACK_UNIFORM_HEIGHT) h=mh;
        else h=child->getDefaultHeight();
        if(galleyheight<h) galleyheight=h;
        }
      }
    galleyleft=border+padleft;
    galleyright=width-border-padright;
    galleytop=border+padtop;
    galleybottom=height-border-padbottom;
    //FXTRACE((100,"horz galleyleft=%d galleyright=%d galleytop=%d galleybottom=%d galleyheight=%d\n",galleyleft,galleyright,galleytop,galleybottom,galleyheight));
    ltop=galleytop;
    lbottom=galleytop+galleyheight;
    lleft=galleyleft;
    rbottom=galleybottom;
    rtop=galleybottom-galleyheight;
    rright=galleyright;
    for(child=getFirst(); child; child=child->getNext()){
      if(child->shown()){
        hints=child->getLayoutHints();
        if(child->isMemberOf(FXMETACLASS(FXToolbarGrip))){
          w=child->getDefaultWidth();
          h=galleyheight;
          }
        else{
          if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth();
          else if(options&PACK_UNIFORM_WIDTH) w=mw;
          else w=child->getDefaultWidth();
          if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
          else if(options&PACK_UNIFORM_HEIGHT) h=mh;
          else h=child->getDefaultHeight();
          }
        if(hints&LAYOUT_RIGHT){
          if(rright-w<galleyleft && rright!=galleyright){
            rbottom=rtop;
            rtop-=galleyheight;
            rright=galleyright;
            }
          x=rright-w;
          rright-=(w+hspacing);
          y=rtop+(galleyheight-h)/2;
          //FXTRACE((100,"horz right : x=%d y=%d w=%d h=%d\n",x,y,w,h));
          }
        else{
          if(lleft+w>galleyright && lleft!=galleyleft){
            ltop=lbottom;
            lbottom+=galleyheight;
            lleft=galleyleft;
            }
          x=lleft;
          lleft+=(w+hspacing);
          y=ltop+(galleyheight-h)/2;
          //FXTRACE((100,"horz left  : x=%d y=%d w=%d h=%d\n",x,y,w,h));
          }
        child->position(x,y,w,h);
        }
      }
    }
  flags&=~FLAG_DIRTY;
  }


// Return true if toolbar is docked
FXbool FXToolbar::isDocked() const { 
  return (getParent()==(FXWindow*)drydock);
  }


// Dock the bar
void FXToolbar::dock(){
  if(wetdock && drydock){
    if(!isDocked()){
      reparent(drydock);
      wetdock->hide();
      }
    }
  }


// Undock the bar
void FXToolbar::undock(){
  if(wetdock && drydock){
    if(isDocked()){
      reparent(wetdock);
      wetdock->resize(wetdock->getDefaultWidth(),wetdock->getDefaultHeight());  // FIXME is needed?
      wetdock->show();
      }
    }
  }


// Save data
void FXToolbar::save(FXStream& store) const {
  FXPacker::save(store);
  store << drydock;
  store << wetdock;
  }


// Load data
void FXToolbar::load(FXStream& store){ 
  FXPacker::load(store);
  store >> drydock;
  store >> wetdock;
  }


// Destroy
FXToolbar::~FXToolbar(){
  drydock=(FXComposite*)-1;
  wetdock=(FXComposite*)-1;
  }
